package com.ether.dao;

import com.ether.PersistenceException;
import com.medias.database.objects.Parametre;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

/**
 * @author vmipsl
 * @date 07 mar 2011
 */
public class ParameterDAOImpl
        extends DomainAccessObjectImpl<Parametre, Integer>
        implements ParameterDAO
{
    protected ParameterDAOImpl()
    {
        super( Parametre.class );
    }

    @NotNull
    public List<Parametre> getParametersByPlateformId( @NotNull final Integer plateformId )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( Parametre.class, "parameter" )
                .createCriteria( "plateformes" )
                .add( Restrictions.eq( "id", plateformId ) )
                .addOrder( Order.asc( "parameter.parametreNom" ) );

        return selectAllByCriteria( Parametre.class, criteria );
    }

    @Nullable
    public Parametre getParameterById( @NotNull final Integer parameterId )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( Parametre.class )
                .add( Restrictions.idEq( parameterId ) );

        return selectByCriteria( Parametre.class, criteria );
    }

    @Nullable
    public List<Parametre> getAllParametersOrderByCategoryByName()
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( Parametre.class, "parameter" )
                .createCriteria( "categorieParam", "categorieParam" )
                .addOrder( Order.asc( "categorieParam.categorieParamNom" ) )
                .addOrder( Order.asc( "parameter.parametreNom" ) );

        return selectAllByCriteria( Parametre.class, criteria );
    }

    @Nullable
    public Integer getParameterByCode( @NotNull final String parameterCode )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( Parametre.class )
                .add( Restrictions.eq( "parametreCode", parameterCode ) )
                .setProjection( Projections.property( "parametreId" ) );

        return selectByCriteria( Integer.class, criteria );
    }
}