package com.ether;

import com.medias.megapoli.utils.MegapoliInitialisation;
import gov.noaa.pmel.sgt.JPane;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Calendar;
import java.util.Locale;
import java.util.ResourceBundle;

/**
 * @author vmipsl
 * @date 05 july 2011
 */
public class EtherPlotServiceImpl
        extends EtherPlotContentServiceImpl
        implements EtherPlotService
{
    /**
     * This method returns the maximum width available to display the quicklook in terms of screenSize and default maximum width
     *
     * @return
     */
    public static Integer getMaxWidth()
    {
        final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        return Math.min( Double.valueOf( screenSize.getWidth() ).intValue(), MAX_WIDTH );
    }

    /**
     * This method returns the maximum height available to display the quicklook in terms of screenSize and default maximum height
     *
     * @return
     */
    public static Integer getMaxHeight()
    {
        final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        return Math.min( Double.valueOf( screenSize.getHeight() ).intValue(), MAX_HEIGHT );
    }

    /**
     * Create the main JPane with the 3 jPanes : top, center (graph) et bottom
     *
     * @param megapoliPlot
     * @param locale
     * @return
     */
    @NotNull
    public BufferedImage createMainPane( @NotNull final MegapoliPlot megapoliPlot, @Nullable final Locale locale )
            throws ServiceException
    {
        final Integer maxWidth = getMaxWidth();
        final Integer maxHeight = getMaxHeight();

        final JPane jPane = new JPane( "Main jPane", new Dimension( maxWidth, maxHeight ) );
        jPane.setLayout( new BorderLayout() );
        jPane.setBackground( Color.white );
        jPane.setOpaque( true );

        // Top Pane
        final JPane jPaneTop = createTopPane( megapoliPlot );
        jPane.add( jPaneTop, BorderLayout.NORTH );

        // Bottom Pane
        final JPane jPaneBottom;
        if( megapoliPlot instanceof SimulationPlot )
            jPaneBottom = createBottomPane( megapoliPlot.getValuesNumber(), ( (SimulationPlot) megapoliPlot ).getRealDataNumber(), locale );
        else
            jPaneBottom = createBottomPane( megapoliPlot.getValuesNumber(), null, locale );
        jPane.add( jPaneBottom, BorderLayout.SOUTH );

        // Center Pane
        final Integer centerHeight = maxHeight - jPaneTop.getHeight() - jPaneBottom.getHeight();
        if( null != megapoliPlot.getValuesNumber() && megapoliPlot.getValuesNumber() > 0 )
        {
            final JPane jPaneCenter = createCenterPane( megapoliPlot, maxWidth, centerHeight, locale );

            if( null == jPaneCenter )
            {
                final JPane emptyJPaneCenter = createEmptyCenterPane( locale );
                jPane.add( emptyJPaneCenter, BorderLayout.CENTER );
            }
            else
            {
                final JLabel jLabelPlot = transformJPaneToJLabel( jPaneCenter );

                final JPane jPaneCenterToImage = new JPane();
                jPaneCenterToImage.setLayout( new GridBagLayout() );
                jPaneCenterToImage.add( jLabelPlot,
                        new GridBagConstraints( 0, 0, 1, 1, 0, 0,
                                GridBagConstraints.CENTER,
                                GridBagConstraints.CENTER,
                                new Insets( 0, 0, 0, 0 ), 0, 0 ) );

                jPane.add( jPaneCenterToImage, BorderLayout.CENTER );
            }
        }
        else
        {
            //throw new ServiceException( ServiceException.ServiceCode.NO_DATA, "No data for this plot", ServiceException.getExceptionThrowable() );
            final JPane jPaneCenter = createEmptyCenterPane( locale );
            jPane.add( jPaneCenter, BorderLayout.CENTER );
        }

        final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPane );
        return jPanelToImageManager.saveAsImage();
    }

    /**
     * Create the top JPane with the logos and title
     *
     * @param megapoliPlot
     * @return
     */
    @NotNull
    public JPane createTopPane( @Nullable final MegapoliPlot megapoliPlot )
    {
        // Logos
        final ImageIcon logoMegapoli = new ImageIcon( megapoliPlot.getLogoMegapoli() );
        final JLabel jLabelLogoMegapoli = new JLabel( logoMegapoli );

        final ImageIcon logoEther = new ImageIcon( megapoliPlot.getLogoEther() );
        final JLabel jLabelLogoEther = new JLabel( logoEther );

        // Max height
        final Integer maxHeights = Math.max( Math.max( logoMegapoli.getIconHeight(), logoEther.getIconHeight() ), MAX_HEIGHT_TOP );

        // jPane
        final JPane jPaneTop = new JPane( "Top jPane", new Dimension( 0, maxHeights ) );
        jPaneTop.setLayout( new BorderLayout() );

        jPaneTop.add( jLabelLogoMegapoli, BorderLayout.LINE_START );
        jPaneTop.add( jLabelLogoEther, BorderLayout.LINE_END );


        // Title
        final String formatedTitle = formatTitle( megapoliPlot.getTitle() );
        final JLabel jLabelTitle = new JLabel( formatedTitle, JLabel.CENTER );
        final Font police = new Font( "Arial", Font.BOLD, TITLE_FONT_SIZE );
        jLabelTitle.setFont( police );
        jPaneTop.add( jLabelTitle, BorderLayout.CENTER );

        return jPaneTop;
    }

    /**
     * Create the bottom JPane with the text "published .. (date)"
     *
     * @param dataNumber
     * @param realDataNumber
     * @param locale
     * @return
     */
    @NotNull
    public JPane createBottomPane( @Nullable final Integer dataNumber, @Nullable final Integer realDataNumber, @Nullable final Locale locale )
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );

        final JPane jPaneBottom = new JPane();
        jPaneBottom.setLayout( new BorderLayout() );

        // Published
        final Calendar calendar = Calendar.getInstance();
        final String messagePublished = bundle.getString( "plot.published" );
        final JLabel jLabel = new JLabel( messagePublished + ' ' + DateHelper.formatDate( calendar.getTime(), DateHelper.ENGLISH_DATE_PATTERN ) + " " );
        final Font police = new Font( "Arial", Font.BOLD, FONT_SIZE );
        jLabel.setFont( police );
        jPaneBottom.add( jLabel, BorderLayout.EAST );

        if( null != dataNumber )
        {
            if( null != realDataNumber )
            {
                final JLabel jLabelDataNumber = new JLabel( bundle.getString( "plot.dataNumber" ) + ' ' + dataNumber + ' ' + bundle.getString( "plot.simulatedDataNumber" ) + ", " + realDataNumber + ' ' + bundle.getString( "plot.realDataNumber" ) );
                jLabelDataNumber.setFont( police );
                jPaneBottom.add( jLabelDataNumber, BorderLayout.WEST );
            }
            else
            {
                final String messageDataNumber = bundle.getString( "plot.dataNumber" );
                final JLabel jLabelDataNumber = new JLabel( messageDataNumber + ' ' + dataNumber );
                jLabelDataNumber.setFont( police );
                jPaneBottom.add( jLabelDataNumber, BorderLayout.WEST );
            }
        }

        return jPaneBottom;
    }

    /**
     * Create an error JPane when plot can be created
     *
     * @param mainError
     * @param errorText
     * @param locale
     * @return
     */
    @NotNull
    public BufferedImage createErrorPane( @NotNull final String mainError, @Nullable final String errorText, @Nullable final Locale locale )
    {
        // Image
        final ImageIcon errorImage = new ImageIcon( MegapoliInitialisation.pathImages + "/erreur.gif" );
        final JLabel jLabelErrorImage = new JLabel( errorImage );

        // Text
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String mainMessage = bundle.getString( "plot.errorMessage" );

        final JLabel jLabelError;
        if( null != errorText )
            jLabelError = new JLabel( "<html>" + mainMessage + "<br><br>" + mainError + "<br>" + errorText + "</html>", JLabel.CENTER );
        else
            jLabelError = new JLabel( "<html>" + mainMessage + "<br><br>" + mainError + "</html>", JLabel.CENTER );

        final Font police = new Font( "Arial", Font.BOLD, ERROR_FONT_SIZE );
        jLabelError.setFont( police );

        // jPane
        final JPane jPane = new JPane( "jPane", new Dimension( MAX_WIDTH, errorImage.getIconHeight() ) );
        jPane.setLayout( new BorderLayout() );
        jPane.setBackground( Color.white );
        jPane.setOpaque( true );
        jPane.add( jLabelErrorImage, BorderLayout.WEST );
        jPane.add( jLabelError, BorderLayout.CENTER );

        final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPane );
        return jPanelToImageManager.saveAsImage();
    }

    /**
     * Create the center JPane with no graph (if no value)
     *
     * @param locale
     * @return
     */
    @NotNull
    public JPane createEmptyCenterPane( @Nullable final Locale locale )
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String messageData = bundle.getString( "plot.noData" );

        final JPane jPaneCenter = new JPane();
        jPaneCenter.setLayout( new BorderLayout() );

        final JLabel jLabelData = new JLabel( messageData, JLabel.CENTER );
        final Font police = new Font( "Arial", Font.BOLD, FONT_SIZE );
        jLabelData.setFont( police );

        jPaneCenter.add( jLabelData, BorderLayout.CENTER );
        return jPaneCenter;
    }

    /**
     * Create the center JPane with the graph
     *
     * @param megapoliPlot
     * @param maxWidth
     * @param maxHeight
     * @return
     */
    @Nullable
    public <T extends MegapoliPlot> JPane createCenterPane( @NotNull final T megapoliPlot, @NotNull final Integer maxWidth, @NotNull final Integer maxHeight, @Nullable final Locale locale )
            throws ServiceException
    {
        final Integer plotWidth = Math.min( maxWidth - MARGIN_LEFT_RIGHT, maxWidth );
        final Integer plotHeight = Math.min( maxHeight - MARGIN_LEFT_RIGHT, maxHeight );

        if( megapoliPlot instanceof VisualisationPlot )
            if( ( (VisualisationPlot) megapoliPlot ).isTimeSerie() )
                return createTimeSeriePlot( (VisualisationPlot) megapoliPlot, plotWidth, plotHeight, locale );
            else
                return create2DPlot( (VisualisationPlot) megapoliPlot, plotWidth, plotHeight, locale );
        else if( megapoliPlot instanceof SimulationPlot )
            return createSimulationPlot( (SimulationPlot) megapoliPlot, plotWidth, plotHeight, locale );
        else
            return null;
    }

    /**
     * This method transform a jPane to a jLabel with a BufferedImage inside and a black border
     *
     * @param jPane
     * @return
     */
    @NotNull
    private JLabel transformJPaneToJLabel( @NotNull final JPane jPane )
    {
        final BufferedImage jPaneBufferedImage = new BufferedImage( jPane.getWidth(), jPane.getHeight(), BufferedImage.TYPE_INT_ARGB );
        final Graphics2D graphics2D = jPaneBufferedImage.createGraphics();
        jPane.addNotify();
        jPane.validate();
        jPane.draw( graphics2D );

        final ImageIcon imageIcon = new ImageIcon( jPaneBufferedImage );
        final JLabel jLabelPlot = new JLabel( imageIcon, JLabel.CENTER );
        jLabelPlot.setBorder( BorderFactory.createLineBorder( Color.black, 1 ) );
        return jLabelPlot;
    }

    @NotNull
    private String formatTitle( @Nullable final String title )
    {
        if( null == title )
            return "";
        else
        {
            final String formatTitle = title.replaceAll( "\\n", "<br>" );
            return "<html>" + formatTitle + "</html>";
        }
    }

    private static final Log LOGGER = LogFactory.getLog( EtherPlotServiceImpl.class );
}
