package com.ether;

import com.ether.dao.JeuDAO;
import com.ether.dao.McoDAO;
import com.ether.dao.MeasureDAO;
import com.ether.dao.ParameterDAO;
import com.ether.dao.PersonDAO;
import com.ether.dao.PlateformDAO;
import com.ether.dao.UserDAO;
import com.ether.dao.ValueDAO;
import com.ether.dao.simulation.AttributeDAO;
import com.ether.dao.simulation.DimensionDAO;
import com.ether.dao.simulation.SimulationDAO;
import com.ether.dao.simulation.VariableDAO;
import com.ether.mco.Mco;
import com.ether.mco.McoFilter;
import com.ether.simulation.Attribute;
import com.ether.simulation.Dimension;
import com.ether.simulation.Simulation;
import com.ether.simulation.SimulationFilter;
import com.ether.simulation.Variable;
import com.ether.user.User;
import com.ether.user.UserFilter;
import com.ether.user.UserState;
import com.medias.annuaire.Personne;
import com.medias.database.objects.Jeu;
import com.medias.database.objects.Parametre;
import com.medias.database.objects.Plateforme;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.transaction.annotation.Transactional;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;
import javax.xml.bind.Unmarshaller;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author vmipsl
 * @date 07 mar 2011
 */
public class EtherServiceImpl
        implements EtherService
{
    @Nullable
    @Transactional(readOnly = true)
    public List<Plateforme> getAllPlateforms()
            throws ServiceException
    {
        try
        {
            return _plateformDAO.getAllPlateforms();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PLATEFORM_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Plateforme getPlateformById( @Nullable final Integer plateformId )
            throws ServiceException
    {
        if( null == plateformId )
            return null;
        try
        {
            return _plateformDAO.getPlateformById( plateformId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PLATEFORM_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Parametre getParameterById( @Nullable final Integer parameterId )
            throws ServiceException
    {
        if( null == parameterId )
            return null;
        try
        {
            return _parameterDAO.getParameterById( parameterId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PARAMETER_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public List<Parametre> getParametersByPlateformId( @NotNull final Integer plateformId )
            throws ServiceException
    {
        try
        {
            return _parameterDAO.getParametersByPlateformId( plateformId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PARAMETER_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public <T1, T2, T3> Data<T1[], T2[], T3[]> getListsByPlateformByParameterByPeriodForTimeSerie( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws ServiceException
    {
        try
        {
            return _valueDAO.getListsByPlateformByParameterByPeriodForTimeSerie( plateformId, parameterId, dateBegin, dateEnd );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VALUE_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public <T1, T2, T3> Data<List<T1>, List<T2>, List<T3>> getListsByPlateformByParameterByPeriodFor2D( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws ServiceException
    {
        try
        {
            return _valueDAO.getListsByPlateformByParameterByPeriodFor2D( plateformId, parameterId, dateBegin, dateEnd );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VALUE_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public <T1, T2, T3> Data<T1[], T2[], T3[]> getArraysByPlateformByParameterByPeriodFor2D( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws ServiceException
    {
        try
        {
            return _valueDAO.getArraysByPlateformByParameterByPeriodFor2D( plateformId, parameterId, dateBegin, dateEnd );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VALUE_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public Integer getNumberValuesByPlateformByParameterByPeriod( @NotNull final List<Pair<Integer, Integer>> pairs, @Nullable final Date beginDate, @Nullable final Date endDate )
            throws ServiceException
    {
        try
        {
            Integer number = 0;
            for( final Pair pair : pairs )
                number += _valueDAO.getNumberValuesByPlateformByParameterByPeriod( (Integer) pair.getFirstValue(), (Integer) pair.getSecondValue(), beginDate, endDate );

            return number;
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VALUE_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Date getFirstDate()
            throws ServiceException
    {
        try
        {
            return _measureDAO.getFirstDate();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.DATE_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Date getLastDate()
            throws ServiceException
    {
        try
        {
            return _measureDAO.getLastDate();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.DATE_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public User getUserById( @NotNull final Integer userId )
            throws ServiceException
    {
        try
        {
            return _userDAO.selectByPrimaryKey( userId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.USER_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public User getUserByEmail( @NotNull final String userEmail )
            throws ServiceException
    {
        try
        {
            return _userDAO.getUserByEmail( userEmail );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.USER_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(rollbackFor = Exception.class)
    public Integer createUser( @NotNull final User user )
            throws ServiceException
    {
        try
        {
            return _userDAO.insert( user );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public List<User> getAllUsers()
            throws ServiceException
    {
        try
        {
            return _userDAO.selectAll();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.USER_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public List<User> getAllUsersByNameOrder()
            throws ServiceException
    {
        try
        {
            return _userDAO.getAllUsersByNameOrder();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.USER_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(readOnly = true)
    public List<User> getUsersByState( @NotNull final UserState userState )
            throws ServiceException
    {
        try
        {
            return _userDAO.getUsersByState( userState );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.USER_NOT_FOUND, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void removeUserById( @NotNull final Integer userId )
            throws ServiceException
    {
        try
        {
            _userDAO.deleteByPrimaryKey( userId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateUser( @NotNull final User user )
            throws ServiceException
    {
        try
        {
            _userDAO.update( user );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void acceptOrRefuseUser( @NotNull final Integer userId, final boolean isAccepted )
            throws ServiceException
    {
        try
        {
            final User user = _userDAO.selectByPrimaryKey( userId );
            if( isAccepted )
                user.setState( UserState.ACCEPTED );
            else
                user.setState( UserState.REFUSED );
            _userDAO.update( user );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Personne getPeopleByEmail( @NotNull final String email )
            throws ServiceException
    {
        try
        {
            return _personDAO.getPersonByEmail( email );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSON_NOT_FOUND, e );
        }
    }

    @Transactional(readOnly = true)
    public PaginatedResult<User> searchUsers( @NotNull final UserFilter filter )
            throws ServiceException
    {
        try
        {
            return _userDAO.search( filter );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.USER_NOT_FOUND, e );
        }
    }

    @Transactional(readOnly = true)
    public PaginatedResult<Mco> searchMcos( @NotNull final McoFilter filter )
            throws ServiceException
    {
        try
        {
            return _mcoDAO.search( filter );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.REQUEST_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Mco getMcoByCode( @NotNull final String code )
            throws ServiceException
    {
        try
        {
            return _mcoDAO.getMcoByCode( code );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.REQUEST_NOT_FOUND, e );
        }
    }

    @NotNull
    @Transactional(rollbackFor = Exception.class)
    public Integer createMco( @NotNull final Mco mco )
            throws ServiceException
    {
        try
        {
            return _mcoDAO.insert( mco );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void removeMcoById( @NotNull final Integer mcoId )
            throws ServiceException
    {
        try
        {
            _mcoDAO.deleteByPrimaryKey( mcoId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateMco( @NotNull final Mco mco )
            throws ServiceException
    {
        try
        {
            _mcoDAO.update( mco );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Mco getMcoById( @NotNull final Integer requestId )
            throws ServiceException
    {
        try
        {
            return _mcoDAO.selectByPrimaryKey( requestId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.REQUEST_NOT_FOUND, e );
        }
    }

    /**
     * This method manage the sub-menus for the list of parameters (to avoid multiple "Particle Concentration" by example)
     * The fullParameters must be ordered with parameterName ascendant
     *
     * @param fullParameters
     * @return
     */
    @NotNull
    public List<List<Parametre>> manageMenusForParameterList( @NotNull final List<Parametre> fullParameters )
    {
        final List<List<Parametre>> parameterListWithMenu = new ArrayList<List<Parametre>>();

        int i = 0;
        while( i < fullParameters.size() )
        {
            final Parametre fullParameter = fullParameters.get( i );
            final int firstIndex = fullParameters.indexOf( fullParameter );
            final int lastIndex = fullParameters.lastIndexOf( fullParameter );
            final List<Parametre> parameters = fullParameters.subList( firstIndex, lastIndex + 1 );
            parameterListWithMenu.add( parameters );
            i += parameters.size();
        }

        return parameterListWithMenu;
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<Parametre> getAllParametersOrderByCategoryByName()
            throws ServiceException
    {
        try
        {
            return _parameterDAO.getAllParametersOrderByCategoryByName();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PARAMETER_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<Date> getAllMeasureDateByPlateformId( @NotNull final Integer plateformId )
            throws ServiceException
    {
        try
        {
            return _measureDAO.getAllMeasureDateByPlateformId( plateformId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.DATE_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public PaginatedResult<Jeu> searchJeux( @NotNull final JeuFilter filter )
            throws ServiceException
    {
        try
        {
            return _jeuDAO.search( filter );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.JEU_NOT_FOUND, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void removeJeuAndDataByName( @NotNull final String datasetName )
            throws ServiceException
    {
        try
        {
            _jeuDAO.removeJeuAndDataByName( datasetName );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void cleanLocalisationAndFlagTables()
            throws ServiceException
    {
        try
        {
            _jeuDAO.cleanLocalisationAndFlagTables();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Jeu getJeuById( @NotNull final Integer jeuId )
            throws ServiceException
    {
        try
        {
            return _jeuDAO.selectByPrimaryKey( jeuId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.JEU_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public String getDatasetNameById( @NotNull final Integer jeuId )
            throws ServiceException
    {
        try
        {
            return _jeuDAO.getDatasetNameById( jeuId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.JEU_NOT_FOUND, e );
        }
    }

    @Nullable
    public PaginatedResult<Dimension> searchDimensions( @NotNull final SimulationFilter filter )
            throws ServiceException
    {
        try
        {
            return _dimensionDAO.search( filter );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.DIMENSION_NOT_FOUND, e );
        }
    }

    @Nullable
    public PaginatedResult<Attribute> searchAttributes( @NotNull final SimulationFilter filter )
            throws ServiceException
    {
        try
        {
            return _attributeDAO.search( filter );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.ATTRIBUTE_NOT_FOUND, e );
        }
    }

    @Nullable
    public PaginatedResult<Variable> searchVariables( @NotNull final SimulationFilter filter )
            throws ServiceException
    {
        try
        {
            return _variableDAO.search( filter );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VARIABLE_NOT_FOUND, e );
        }
    }

    @Nullable
    public PaginatedResult<Simulation> searchSimulations( @NotNull final SimulationFilter filter )
            throws ServiceException
    {
        try
        {
            return _simulationDAO.search( filter );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.SIMULATION_NOT_FOUND, e );
        }
    }

    /**
     * This method create a directory for all ".nc" files in the simulation directory
     * And create for each a "header_init.xml" file which contents the attributes, variables, dimensions, etc... for the simulation
     * The second line of the "header_init.xml" file is changed by "xmlnsHeader" because of the term "xmlns" which is not ok for the "unmarshall" method
     * The real file to work becomes "header.xml"
     *
     * @param simulationPath
     */
    public void launchNCDumpInSimulationDirectory( @NotNull final String simulationPath )
            throws ServiceException
    {
        final File simulationsDirectory = new File( simulationPath );
        for( final String fileName : simulationsDirectory.list() )
        {
            if( fileName.endsWith( FORMAT_FILE ) )
            {
                // Create directory and move simulation file into
                final String shortFileName = fileName.substring( 0, fileName.lastIndexOf( '.' ) );
                final String directoryName = simulationPath + File.separatorChar + shortFileName;
                final File directory = new File( directoryName );
                final File file = new File( simulationPath + File.separatorChar + fileName );
                if( !directory.exists() )
                {
                    directory.mkdir();
                    file.renameTo( new File( directoryName + File.separatorChar + fileName ) );
                    try
                    {
                        final Runtime runtime = Runtime.getRuntime();
                        final String headerInit = directoryName + File.separatorChar + "header_init.xml";
                        final String[] argsNcdump = {"/bin/sh", "-c", "/usr/bin/ncdump -h -x " + directoryName + File.separatorChar + fileName + " > " + headerInit};
                        runtime.exec( argsNcdump );
                        EtherHelper.waitForFileWithMax( new File( headerInit ), 5000 );
                        final String[] argsSed = {"/bin/sh", "-c", "sed s/xmlns/xmlnsHeader/ " + directoryName + File.separatorChar + "header_init.xml > " + directoryName + File.separatorChar + FILE_HEADER_NAME};
                        runtime.exec( argsSed );
                    }
                    catch( IOException e )
                    {
                        throw new ServiceException( ServiceException.ServiceCode.ERROR_PROCESS_SIMULATION_NCDUMP, e );
                    }
                    catch( InterruptedException e )
                    {
                        throw new ServiceException( ServiceException.ServiceCode.ERROR_PROCESS_SIMULATION_NCDUMP_WAIT, e );
                    }
                }
            }
        }
    }

    /**
     * This method insert the header and the dates (variable "Times") in database
     * The dates are saved in database as a unique String (use to split when we need to have the real values)
     * The new variables are separatly created in the table ....
     * The metadata are saved in the table ....
     *
     * @param simulationPath
     * @param directoryName
     */
    @Transactional(rollbackFor = Exception.class)
    public void insertHeader( @NotNull final String simulationPath, @NotNull final String directoryName )
            throws ServiceException
    {
        try
        {
            final String fileName = simulationPath + File.separatorChar + directoryName + File.separatorChar + FILE_HEADER_NAME;

            final JAXBContext context = JAXBContext.newInstance( Simulation.class );
            final Marshaller marshaller = context.createMarshaller();
            marshaller.setProperty( Marshaller.JAXB_FORMATTED_OUTPUT, true );
            final Unmarshaller uMarshaller = context.createUnmarshaller();
            final Simulation simulation = (Simulation) uMarshaller.unmarshal( new File( fileName ) );

            // Extract dates for the simulation
            final Runtime runtime = Runtime.getRuntime();
            final String[] argsNcdump = {"/bin/sh", "-c", "ncdump -v Times " + simulationPath + File.separatorChar + directoryName + File.separatorChar + directoryName + FORMAT_FILE + " | sed -e '1,/data:/d' -e '1,/Times/d' -e '$d'"};
            final Process exec = runtime.exec( argsNcdump );

            final BufferedReader bufferedReader = new BufferedReader( new InputStreamReader( exec.getInputStream() ) );
            final StringBuffer stringBuffer = new StringBuffer();
            String line;
            while( ( line = bufferedReader.readLine() ) != null )
                stringBuffer.append( line );

            final String dates = stringBuffer.toString().replaceAll( "\"", "" ).replaceAll( " ", "" ).replaceAll( ";", "" );
            simulation.setDates( dates );

            final Integer simulationId = insertSimulation( simulation );
        }
        catch( PropertyException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.ERROR_PROPERTY, e );
        }
        catch( JAXBException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.ERROR_JAXB, e );
        }
        catch( IOException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.ERROR_PROCESS_SIMULATION_NCDUMP, e );
        }
    }

    /**
     * This method extract data (parameters, latitudes, longitutes) from the file corresponding to the simulation for the megapoliPlot
     * We use ncks to extract only data for a geographic area and a date
     * http://linux.die.net/man/1/ncks
     * Example to see easily : ncks -H -d Time,0 -d bottom_top,0 -v APINEN out.20090701_20090705_01_SMPV4_complet.nc
     *
     * @param modelId
     * @param variableName
     * @param dateNumber            : the number of the date among all the available dates
     * @param level
     * @param writeInTemporaryFiles : use to debug and see the result of the process
     * @return
     * @throws IOException
     * @throws ServiceException
     */
    @Nullable
    @Transactional(readOnly = true)
    public List<Double> extractValuesByModelByVariableByDateByLevelFromFile( @NotNull final Integer modelId, @NotNull final String variableName, @NotNull final Integer dateNumber, @NotNull final Integer level, final boolean writeInTemporaryFiles )
            throws ServiceException
    {
        final String filePath = getLocationByModelId( modelId );
        final String resultFile = filePath + '_' + variableName;

        try
        {
            final Runtime runtime = Runtime.getRuntime();
            // TODO : voir pour la précision -s "%f\n"
            final String processString = "ncks -s \"%16.30f\\n\" -d Time," + dateNumber + " -d bottom_top," + level + " -v " + variableName + ' ' + filePath + " | sed '/^" + variableName + "/d'";
            final String[] argsNcdump = {"/bin/sh", "-c", processString};
            final Process exec = runtime.exec( argsNcdump );

            if( writeInTemporaryFiles )
                runtime.exec( new String[]{"/bin/sh", "-c", processString + " > " + resultFile} );

            // TODO : voir avec java 7 pour utiliser le Files.readAllLines() et obtenir directement une List<>, http://adiguba.developpez.com/tutoriels/java/7/
            final BufferedReader bufferedReader = new BufferedReader( new InputStreamReader( exec.getInputStream() ) );
            final List<Double> valuesList = new ArrayList<Double>();
            String line;
            while( ( line = bufferedReader.readLine() ) != null )
                if( !line.isEmpty() )
                    valuesList.add( Double.valueOf( line ) );

            return valuesList;
        }
        catch( IOException e )
        {
            if( new File( resultFile ).exists() )
                new File( resultFile ).delete();

            throw new ServiceException( ServiceException.ServiceCode.ERROR_PROCESS_SIMULATION_NCDUMP, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Dimension getDimensionById( @NotNull final Integer dimensionId )
            throws ServiceException
    {
        try
        {
            return _dimensionDAO.selectByPrimaryKey( dimensionId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.DIMENSION_NOT_FOUND, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void removeDimension( @NotNull final Integer dimensionId )
            throws ServiceException
    {
        try
        {
            _dimensionDAO.deleteByPrimaryKey( dimensionId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Attribute getAttributeById( @NotNull final Integer attributeId )
            throws ServiceException
    {
        try
        {
            return _attributeDAO.selectByPrimaryKey( attributeId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.ATTRIBUTE_NOT_FOUND, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void removeAttribute( @NotNull final Integer attributeId )
            throws ServiceException
    {
        try
        {
            _attributeDAO.deleteByPrimaryKey( attributeId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Variable getVariableById( @NotNull final Integer variableId )
            throws ServiceException
    {
        try
        {
            return _variableDAO.selectByPrimaryKey( variableId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VARIABLE_NOT_FOUND, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void removeVariable( @NotNull final Integer variableId )
            throws ServiceException
    {
        try
        {
            _variableDAO.deleteByPrimaryKey( variableId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Simulation getSimulationById( @NotNull final Integer simulationId )
            throws ServiceException
    {
        try
        {
            return _simulationDAO.selectByPrimaryKey( simulationId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.SIMULATION_NOT_FOUND, e );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void removeSimulation( @NotNull final Integer simulationId )
            throws ServiceException
    {
        try
        {
            _simulationDAO.deleteByPrimaryKey( simulationId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<Simulation> getAllSimulations()
            throws ServiceException
    {
        try
        {
            return _simulationDAO.selectAll();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.SIMULATION_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<List<String>> getAllModels()
            throws ServiceException
    {
        try
        {
            return _simulationDAO.getAllModels();
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.MODEL_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<Variable> getVariablesByModelId( @NotNull final Integer modelId )
            throws ServiceException
    {
        try
        {
            return _variableDAO.getVariablesByModelId( modelId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.VARIABLE_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public String getLocationByModelId( @NotNull final Integer modelId )
            throws ServiceException
    {
        try
        {
            return _simulationDAO.getLocationByModelId( modelId );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.SIMULATION_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public List<Plateforme> getPlateformsByParameter( @NotNull final String parameterCode )
            throws ServiceException
    {
        try
        {
            return _plateformDAO.getPlateformsByParameter( parameterCode );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PLATEFORM_NOT_FOUND, e );
        }
    }

    @Nullable
    @Transactional(readOnly = true)
    public Integer getParameterByCode( @NotNull final String parameterCode )
            throws ServiceException
    {
        try
        {
            return _parameterDAO.getParameterByCode( parameterCode );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PARAMETER_NOT_FOUND, e );
        }
    }

    /**
     * This method insert in database the new dimensions, attributes, variables and simulation
     * We use HashSet to avoid doublon ( no new insertion for a parameter already in DB )
     *
     * @param simulation
     * @return
     * @throws ServiceException
     */
    private Integer insertSimulation( final Simulation simulation )
            throws ServiceException
    {
        try
        {
            // Insert dimensions
            final Set<Dimension> dimensionsInDB = new HashSet<Dimension>( _dimensionDAO.selectAll() );
            final Set<Dimension> dimensionsInSimulation = simulation.getDimension();
            final Set<Dimension> dimensionsInDBAndSimulation = new HashSet<Dimension>( dimensionsInDB );
            dimensionsInDBAndSimulation.retainAll( dimensionsInSimulation );
            _dimensionDAO.updateAll( dimensionsInDBAndSimulation );
            final Set<Dimension> dimensionsToAdd = new HashSet<Dimension>( dimensionsInSimulation );
            dimensionsToAdd.removeAll( dimensionsInDB );
            _dimensionDAO.insertAll( dimensionsToAdd );
            dimensionsInDBAndSimulation.addAll( dimensionsToAdd );
            simulation.setDimension( dimensionsInDBAndSimulation );

            // Insert Attributes
            final Set<Attribute> attributesInDB = new HashSet<Attribute>( _attributeDAO.selectAll() );
            final Set<Attribute> attributesInSimulation = simulation.getAttribute();
            final Set<Attribute> attributesInDBAndSimulation = new HashSet<Attribute>( attributesInDB );
            attributesInDBAndSimulation.retainAll( attributesInSimulation );
            _attributeDAO.updateAll( attributesInDBAndSimulation );
            final Set<Attribute> attributesToAdd = new HashSet<Attribute>( attributesInSimulation );
            attributesToAdd.removeAll( attributesInDB );
            _attributeDAO.insertAll( attributesToAdd );
            attributesInDBAndSimulation.addAll( attributesToAdd );
            simulation.setAttribute( attributesInDBAndSimulation );

            // Insert variables
            final Set<Variable> variablesInDB = new HashSet<Variable>( _variableDAO.selectAll() );
            final Set<Variable> variablesInSimulation = simulation.getVariable();
            final Set<Variable> variablesInDBAndSimulation = new HashSet<Variable>( variablesInDB );
            variablesInDBAndSimulation.retainAll( variablesInSimulation );
            _variableDAO.updateAll( variablesInDBAndSimulation );
            final Set<Variable> variablesToAdd = new HashSet<Variable>( variablesInSimulation );
            variablesToAdd.removeAll( variablesInDB );
            _variableDAO.insertAll( variablesToAdd );
            variablesInDBAndSimulation.addAll( variablesToAdd );
            simulation.setVariable( variablesInDBAndSimulation );

            // Insert header
            final Simulation simulationInDB = _simulationDAO.getSimulationByLocation( simulation.getLocation() );
            if( null != simulationInDB )
            {
                simulationInDB.setXmlnsHeader( simulation.getXmlnsHeader() );
                simulationInDB.setDimension( simulation.getDimension() );
                simulationInDB.setAttribute( simulation.getAttribute() );
                simulationInDB.setVariable( simulation.getVariable() );
                _simulationDAO.update( simulationInDB );
                return simulationInDB.getId();
            }
            else
                return _simulationDAO.insert( simulation );
        }
        catch( PersistenceException e )
        {
            throw new ServiceException( ServiceException.ServiceCode.PERSISTENCE, e );
        }
    }

    @Required
    public void setPlateformDAO( final PlateformDAO plateformDAO )
    {
        _plateformDAO = plateformDAO;
    }

    @Required
    public void setParameterDAO( final ParameterDAO parameterDAO )
    {
        _parameterDAO = parameterDAO;
    }

    @Required
    public void setValueDAO( final ValueDAO valueDAO )
    {
        _valueDAO = valueDAO;
    }

    @Required
    public void setMeasureDAO( final MeasureDAO measureDAO )
    {
        _measureDAO = measureDAO;
    }

    @Required
    public void setUserDAO( final UserDAO userDAO )
    {
        _userDAO = userDAO;
    }

    @Required
    public void setPersonDAO( final PersonDAO personDAO )
    {
        _personDAO = personDAO;
    }

    @Required
    public void setMcoDAO( final McoDAO mcoDAO )
    {
        _mcoDAO = mcoDAO;
    }

    @Required
    public void setJeuDAO( final JeuDAO jeuDAO )
    {
        _jeuDAO = jeuDAO;
    }

    @Required
    public void setDimensionDAO( final DimensionDAO dimensionDAO )
    {
        _dimensionDAO = dimensionDAO;
    }

    @Required
    public void setAttributeDAO( final AttributeDAO attributeDAO )
    {
        _attributeDAO = attributeDAO;
    }

    @Required
    public void setVariableDAO( final VariableDAO variableDAO )
    {
        _variableDAO = variableDAO;
    }

    @Required
    public void setSimulationDAO( final SimulationDAO simulationDAO )
    {
        _simulationDAO = simulationDAO;
    }

    private static final Log LOGGER = LogFactory.getLog( EtherServiceImpl.class );

    private static String FILE_HEADER_NAME = "header.xml";
    private static String FORMAT_FILE = ".nc";

    private PlateformDAO _plateformDAO;
    private ParameterDAO _parameterDAO;
    private ValueDAO _valueDAO;
    private MeasureDAO _measureDAO;
    private UserDAO _userDAO;
    private PersonDAO _personDAO;
    private McoDAO _mcoDAO;
    private JeuDAO _jeuDAO;
    private DimensionDAO _dimensionDAO;
    private AttributeDAO _attributeDAO;
    private VariableDAO _variableDAO;
    private SimulationDAO _simulationDAO;
}
