package com.ether;

import com.ether.mco.Mco;
import com.ether.mco.McoFilter;
import com.ether.simulation.Attribute;
import com.ether.simulation.Dimension;
import com.ether.simulation.Simulation;
import com.ether.simulation.SimulationFilter;
import com.ether.simulation.Variable;
import com.ether.user.User;
import com.ether.user.UserFilter;
import com.ether.user.UserState;
import com.medias.annuaire.Personne;
import com.medias.database.objects.Jeu;
import com.medias.database.objects.Parametre;
import com.medias.database.objects.Plateforme;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Date;
import java.util.List;

/**
 * @author vmipsl
 * @date 07 mar 2011
 */
public interface EtherService
        extends Service
{
    @Nullable
    List<Plateforme> getAllPlateforms()
            throws ServiceException;

    @Nullable
    Plateforme getPlateformById( @Nullable final Integer plateformId )
            throws ServiceException;

    @Nullable
    Parametre getParameterById( @Nullable final Integer parameterId )
            throws ServiceException;

    @NotNull
    List<Parametre> getParametersByPlateformId( @NotNull final Integer plateformId )
            throws ServiceException;

    @NotNull
    <T1, T2, T3> Data<T1[], T2[], T3[]> getListsByPlateformByParameterByPeriodForTimeSerie( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws ServiceException;

    @NotNull
    <T1, T2, T3> Data<List<T1>, List<T2>, List<T3>> getListsByPlateformByParameterByPeriodFor2D( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws ServiceException;

    @NotNull
    <T1, T2, T3> Data<T1[], T2[], T3[]> getArraysByPlateformByParameterByPeriodFor2D( @NotNull final Integer plateformId, @NotNull final Integer parameterId, @Nullable final Date dateBegin, @Nullable final Date dateEnd )
            throws ServiceException;

    @NotNull
    Integer getNumberValuesByPlateformByParameterByPeriod( @NotNull final List<Pair<Integer, Integer>> pairs, @Nullable final Date beginDate, @Nullable final Date endDate )
            throws ServiceException;

    @Nullable
    Date getFirstDate()
            throws ServiceException;

    @Nullable
    Date getLastDate()
            throws ServiceException;

    @Nullable
    User getUserById( @NotNull final Integer userId )
            throws ServiceException;

    @Nullable
    User getUserByEmail( @NotNull final String userEmail )
            throws ServiceException;

    @NotNull
    Integer createUser( @NotNull User user )
            throws ServiceException;

    @NotNull
    List<User> getAllUsers()
            throws ServiceException;

    @NotNull
    List<User> getAllUsersByNameOrder()
            throws ServiceException;

    @NotNull
    List<User> getUsersByState( @NotNull UserState userState )
            throws ServiceException;

    void removeUserById( @NotNull final Integer userId )
            throws ServiceException;

    void updateUser( @NotNull final User user )
            throws ServiceException;

    void acceptOrRefuseUser( @NotNull final Integer userId, final boolean isAccepted )
            throws ServiceException;

    @Nullable
    Personne getPeopleByEmail( @NotNull final String email )
            throws ServiceException;

    PaginatedResult<User> searchUsers( @NotNull final UserFilter filter )
            throws ServiceException;

    PaginatedResult<Mco> searchMcos( @NotNull final McoFilter filter )
            throws ServiceException;

    @Nullable
    Mco getMcoByCode( @NotNull final String code )
            throws ServiceException;

    @NotNull
    Integer createMco( @NotNull Mco mco )
            throws ServiceException;

    void removeMcoById( @NotNull final Integer mcoId )
            throws ServiceException;

    void updateMco( @NotNull final Mco mco )
            throws ServiceException;

    @Nullable
    Mco getMcoById( @NotNull final Integer requestId )
            throws ServiceException;

    @NotNull
    List<List<Parametre>> manageMenusForParameterList( @NotNull final List<Parametre> fullParameters );

    @Nullable
    List<Parametre> getAllParametersOrderByCategoryByName()
            throws ServiceException;

    @Nullable
    List<Date> getAllMeasureDateByPlateformId( @NotNull final Integer plateformId )
            throws ServiceException;

    @Nullable
    PaginatedResult<Jeu> searchJeux( @NotNull final JeuFilter filter )
            throws ServiceException;

    void removeJeuAndDataByName( @NotNull final String datasetName )
            throws ServiceException;

    void cleanLocalisationAndFlagTables()
            throws ServiceException;

    @Nullable
    Jeu getJeuById( @NotNull final Integer jeuId )
            throws ServiceException;

    @Nullable
    String getDatasetNameById( @NotNull final Integer jeuId )
            throws ServiceException;

    @Nullable
    PaginatedResult<Dimension> searchDimensions( @NotNull final SimulationFilter filter )
            throws ServiceException;

    @Nullable
    PaginatedResult<Attribute> searchAttributes( @NotNull final SimulationFilter filter )
            throws ServiceException;

    @Nullable
    PaginatedResult<Variable> searchVariables( @NotNull final SimulationFilter filter )
            throws ServiceException;

    @Nullable
    PaginatedResult<Simulation> searchSimulations( @NotNull final SimulationFilter filter )
            throws ServiceException;

    public void launchNCDumpInSimulationDirectory( @NotNull final String simulationPath )
            throws ServiceException;

    void insertHeader( @NotNull final String simulationPath, @NotNull final String directoryName )
            throws ServiceException;

    @Nullable
    List<Double> extractValuesByModelByVariableByDateByLevelFromFile( @NotNull final Integer modelId, @NotNull final String variableName, @NotNull final Integer dateNumber, @NotNull final Integer level, final boolean writeInTemporaryFiles )
            throws ServiceException;

    @Nullable
    Dimension getDimensionById( @NotNull final Integer dimensionId )
            throws ServiceException;

    public void removeDimension( @NotNull final Integer dimensionId )
            throws ServiceException;

    @Nullable
    Attribute getAttributeById( @NotNull final Integer attributeId )
            throws ServiceException;

    public void removeAttribute( @NotNull final Integer attributeId )
            throws ServiceException;

    @Nullable
    Variable getVariableById( @NotNull final Integer variableId )
            throws ServiceException;

    public void removeVariable( @NotNull final Integer variableId )
            throws ServiceException;

    @Nullable
    Simulation getSimulationById( @NotNull final Integer simulationId )
            throws ServiceException;

    public void removeSimulation( @NotNull final Integer simulationId )
            throws ServiceException;

    @Nullable
    List<Simulation> getAllSimulations()
            throws ServiceException;

    @Nullable
    List<List<String>> getAllModels()
            throws ServiceException;

    @Nullable
    List<Variable> getVariablesByModelId( @NotNull final Integer modelId )
            throws ServiceException;

    @Nullable
    String getLocationByModelId( @NotNull final Integer modelId )
            throws ServiceException;

    @Nullable
    List<Plateforme> getPlateformsByParameter( @NotNull final String parameterCode )
            throws ServiceException;

    @Nullable
    Integer getParameterByCode( @NotNull final String parameterCode )
            throws ServiceException;
}
