package com.ether;

import com.ether.annotation.ControllerMethod;
import com.ether.annotation.Mandatory;
import com.ether.annotation.ParamName;
import com.ether.simulation.Variable;
import com.medias.database.objects.Plateforme;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author vmipsl
 * @date 17 feb 2011
 */
public class ControllerSimulation
        extends ControllerEther
{
    /** *********************************************************** **/
    /** *********************** VIEWS ***************************** **/
    /** *********************************************************** **/
    /**
     * Default view if methodName is unknown
     */
    public ModelAndView home( final HttpServletRequest request, final HttpServletResponse response )
            throws ServiceException
    {
        return new ModelAndView( VIEW_ERRORS );
    }

    @ControllerMethod(view = VIEW_SIMULATION)
    public Map<String, Object> viewSimulation()
            throws ServiceException
    {
        final List<List<String>> models = getEtherService().getAllModels();

        final Map<String, Object> model = new HashMap<String, Object>();
        model.put( "simulations", getJSONModels( models ) );
        return model;
    }

    /** *********************************************************** **/
    /** *********************** CALLS ***************************** **/
    /** *********************************************************** **/
    /**
     * This method select the molecules available for the given model
     *
     * @param modelId
     * @return
     * @throws ServiceException
     */
    @ControllerMethod(jsonResult = true)
    public JSONObject searchVariablesByModel( @Mandatory @ParamName(ParameterConstants.PARAMETER_ID) final Integer modelId )
            throws ServiceException
    {
        final List<Variable> variablesByModel = getEtherService().getVariablesByModelId( modelId );

        final JSONObject result = new JSONObject();
        result.put( ParameterConstants.PARAMETER_VARIABLES, getJsonHelper().toJSON( variablesByModel ) );
        return result;
    }

    @ControllerMethod(jsonResult = true)
    public JSONObject searchMobilePlateformsByVariable( @Mandatory @ParamName(ParameterConstants.PARAMETER_CODE) final String variableCode )
            throws ServiceException
    {
        final List<Plateforme> plateformsByParameter = getEtherService().getPlateformsByParameter( variableCode );

        final List<Plateforme> mobilePlateforms = new ArrayList<Plateforme>();
        for( final Plateforme plateforme : plateformsByParameter )
        {
            if( "MOBILE".equals( plateforme.getTypePlateforme().getTypePlateformeNom() ) )
                mobilePlateforms.add( plateforme );
        }

        final JSONObject result = new JSONObject();
        result.put( ParameterConstants.PARAMETER_PLATEFORMS, getJsonHelper().toJSON( mobilePlateforms ) );
        return result;
    }

    private List<JSONObject> getJSONModels( @NotNull final List<List<String>> models )
    {
        final List<JSONObject> jsonArray = new ArrayList<JSONObject>();
        for( final List<String> model : models )
        {
            final JSONObject jsonModel = new JSONObject();
            jsonModel.put( "id", model.get( 0 ) );
            jsonModel.put( "name", model.get( 1 ) );
            jsonModel.put( "version", model.get( 2 ) );
            jsonModel.put( "dates", model.get( 3 ).split( "," ) );
            jsonModel.put( "levels", createLevels( Integer.parseInt( model.get( 4 ) ) ) );
            jsonArray.add( jsonModel );
        }
        return jsonArray;
    }

    private List<Integer> createLevels( @NotNull final Integer levelMax )
    {
        final List<Integer> levels = new ArrayList<Integer>();
        for( int i = 1; i <= levelMax; i++ )
            levels.add( i );
        return levels;
    }

    private static final Log LOGGER = LogFactory.getLog( ControllerSimulation.class );

    private static final String VIEW_SIMULATION = "visualization/visu_simulation";
}