package com.ether;

import com.medias.Context;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import javax.imageio.ImageIO;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * @author vmipsl
 * @date 04 september 2012
 */
public class ControllerSimulationPlot
        extends HttpServlet
{
    public void init( final ServletConfig servletConfig )
            throws ServletException
    {
        try
        {
            final ApplicationContext appContext = WebApplicationContextUtils.getRequiredWebApplicationContext( servletConfig.getServletContext() );
            _etherPlotService = (EtherPlotService) appContext.getBean( "etherPlotService", EtherPlotService.class );
            _etherService = (EtherService) appContext.getBean( "etherService", EtherService.class );
        }
        catch( Throwable tx )
        {
            LOGGER.error( "Error initializing EtherService.", tx );
            throw new ServletException( "Error initializing EtherService.", tx );
        }
    }

    /**
     * Creates an image with the plot and send it to the response
     * Call by a jsp with <img src="simulation/plotSimulationEther?....">
     *
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     */
    public void doGet( final HttpServletRequest request, final HttpServletResponse response )
            throws ServletException
    {
        try
        {
            final String title = request.getParameter( ParameterConstants.PARAMETER_TITLE );
            final Integer modelId = Integer.valueOf( request.getParameter( ParameterConstants.PARAMETER_MODEL_ID ) );
            final Integer variableId = Integer.valueOf( request.getParameter( ParameterConstants.PARAMETER_VARIABLE_ID ) );
            final String variableName = request.getParameter( ParameterConstants.PARAMETER_VARIABLE_NAME );
            final Integer dateNumber = Integer.parseInt( request.getParameter( ParameterConstants.PARAMETER_DATE_NUMBER ) );
            final String date = request.getParameter( ParameterConstants.PARAMETER_DATE );
            final Integer level = Integer.parseInt( request.getParameter( ParameterConstants.PARAMETER_VARIABLE_LEVEL ) );
            final boolean isRasterContour = Boolean.parseBoolean( request.getParameter( "isRasterContour" ) );
            final boolean displayRealData = Boolean.parseBoolean( request.getParameter( "isRealDataToDisplay" ) );

            Integer plateformId = null;
            if( null != request.getParameter( ParameterConstants.PARAMETER_PLATEFORM_ID ) )
                plateformId = Integer.parseInt( request.getParameter( ParameterConstants.PARAMETER_PLATEFORM_ID ) );

            // Create plot
            final SimulationPlot megapoliPlot = new SimulationPlot();
            megapoliPlot.setTitle( title );
            megapoliPlot.setModelId( modelId );
            megapoliPlot.setVariableId( variableId );
            megapoliPlot.setVariableName( variableName );
            megapoliPlot.setLevel( level );
            megapoliPlot.setDate( date );
            megapoliPlot.setRasterContour( isRasterContour );
            megapoliPlot.setRealDataToDisplay( displayRealData );

            final List<Double> parameterValues = _etherService.extractValuesByModelByVariableByDateByLevelFromFile( megapoliPlot.getModelId(), megapoliPlot.getVariableName(), dateNumber, level, true );
            final List<Double> latitudeValues = _etherService.extractValuesByModelByVariableByDateByLevelFromFile( megapoliPlot.getModelId(), "lat", dateNumber, level, true );
            final List<Double> longitudeValues = _etherService.extractValuesByModelByVariableByDateByLevelFromFile( megapoliPlot.getModelId(), "lon", dateNumber, level, true );

            final Data valuesLists;
            if( isRasterContour )
            {
                final Set<Double> latitudeValuesSet = new LinkedHashSet<Double>( latitudeValues );
                final Set<Double> longitudeValuesSet = new LinkedHashSet<Double>( longitudeValues );
                valuesLists = new Data( parameterValues, new ArrayList<Double>( latitudeValuesSet ), new ArrayList<Double>( longitudeValuesSet ) );
            }
            else
                valuesLists = new Data( parameterValues, new ArrayList<Double>( latitudeValues ), new ArrayList<Double>( longitudeValues ) );
            megapoliPlot.setData( valuesLists );
            megapoliPlot.setValuesNumber( parameterValues.size() );

            if( displayRealData && null != plateformId )
            {
                final Integer parameterId = _etherService.getParameterByCode( variableName );
                // Dates
                final Calendar calendar = Calendar.getInstance();
                final Date formatedDateBegin = DateHelper.parseDate( date, "yyyy-MM-dd" );
                calendar.setTime( formatedDateBegin );
                calendar.add( Calendar.DAY_OF_YEAR, 1 );
                final Date formatedDateEnd = calendar.getTime();

                final Data realValuesLists = _etherService.getListsByPlateformByParameterByPeriodFor2D( plateformId, parameterId, formatedDateBegin, formatedDateEnd );
                megapoliPlot.setRealData( realValuesLists );
                megapoliPlot.setRealDataNumber( ( (ArrayList) realValuesLists.getFirstArray() ).size() );
            }

            final BufferedImage bufferedImage = _etherPlotService.createMainPane( megapoliPlot, Context.getLocale( request ) );
            ImageIO.write( bufferedImage, "png", response.getOutputStream() );
        }
        catch( Exception e1 )
        {
            try
            {
                createErrorPane( response, request, "", e1 );
            }
            catch( Exception e2 )
            {
                throw new ServletException( "Error : no possibity to write image in response", e2 );
            }
        }
    }

    private void createErrorPane( final HttpServletResponse response, final HttpServletRequest request, @NotNull final String mainError, @Nullable final Exception e )
            throws IOException
    {
        final BufferedImage errorImage;
        if( null != e )
            errorImage = _etherPlotService.createErrorPane( mainError, e.toString(), Context.getLocale( request ) );
        else
            errorImage = _etherPlotService.createErrorPane( mainError, null, Context.getLocale( request ) );

        ImageIO.write( errorImage, "png", response.getOutputStream() );
    }

    private static final Log LOGGER = LogFactory.getLog( ControllerSimulationPlot.class );

    private EtherPlotService _etherPlotService;
    private EtherService _etherService;
}