<%@ page import="com.ether.EtherPlotServiceImpl" %>
<script type="text/javascript">
var InterfaceSimulation = Class.create( {

    initialize: function( jsonSimulations )
    {
        // Values
        this.jsonSimulations = jsonSimulations || null;
        this.selectedModel = false;
        this.selectedVariable = false;
        this.beginDate = false;
        this.endDate = false;

        /** *********** CONTAINERS *********** **/
        this.containerModels = $( "#containerModels" );
        this.containerVariables = $( "#containerVariables" );
        this.containerButtons = $( "#containerButtons" );
        this.containerOptionDates = $( "#containerOptionDates" );
        this.containerOptionLevels = $( "#containerOptionLevels" );
        this.containerOptionTypes = $( "#containerOptionTypes" );
        this.containerPlateforms = $( "#containerSimulationPlateforms" );

        /** *********** LOADING *********** **/
        // Create loading object for plateforms
        var param = new Object();
        param.parent = this.containerModels;
        this.loadingModels = new Loading( param );
        this.loadingModels.display();

        param.parent = this.containerVariables;
        this.loadingVariables = new Loading( param );
        this.loadingVariables.display();

        param.parent = this.containerPlateforms;
        this.loadingPlateforms = new Loading( param );

        this.createOptions();
        this.displayModels();
    },

    // CREATES ********************************************************
    createOptions: function()
    {
        // Type select
        this.containerOptionTypes.empty();
        var paramSelect = new Object();
        paramSelect.parent = this.containerOptionTypes;
        this.selectTypes = new Select( paramSelect );
        this.selectTypes.add( false, interfaceTexts["simulation.raster"] );
        this.selectTypes.add( true, interfaceTexts["simulation.raster_contour"] );
        this.selectTypes.selectFirst( false );

        // Bind checkbox displayRealData
        $( "#displayRealData" ).live( "change", jQuery.proxy( this.onCheckRealData, this ) );
        $( "#displayRealData" ).attr( "disabled", true );

        // Visualize Button
        this.visualizeButton = new Button( {value:interfaceTexts["simulation.button.visualize"], parent:this.containerButtons, id:"button_visualize", onClick:jQuery.proxy( this.onClickVisualize, this )} );
        this.visualizeButton.disable();
    },

    createSelectDates:function()
    {
        if( !this.selectedModel )
            return;

        this.containerOptionDates.empty();

        var paramSelect = new Object();
        paramSelect.parent = this.containerOptionDates;
        this.selectDates = new Select( paramSelect );
        jQuery.each( this.selectedModel.jsonElement.dates, jQuery.proxy( function ( i, date )
        {
            this.selectDates.add( i, date );
        }, this ) );
        this.selectDates.selectFirst( false );
    },

    createSelectLevels: function()
    {
        if( !this.selectedModel )
            return;

        this.containerOptionLevels.empty();

        var paramSelect = new Object();
        paramSelect.parent = this.containerOptionLevels;
        this.selectLevels = new Select( paramSelect );
        jQuery.each( this.selectedModel.jsonElement.levels, jQuery.proxy( function ( i, level )
        {
            this.selectLevels.add( i, level );
        }, this ) );
        this.selectLevels.selectFirst( false );
    },

    // REQUESTS ********************************************************
    requestVariablesByModel: function()
    {
        if( this.selectedModel )
            $.ajax( {
                url: "simulation?methodName=searchVariablesByModel&id=" + this.selectedModel.getId(),
                success:jQuery.proxy( this.handleVariablesBySelectedModel, this )
            } );
    },

    requestPlateformsByVariable: function()
    {
        this.loadingPlateforms.display();
        if( this.selectedVariable )
            $.ajax( {
                url: "simulation?methodName=searchMobilePlateformsByVariable&code=" + this.selectedVariable.getName(),
                success:jQuery.proxy( this.handlePlateformsByVariable, this )
            } );
    },

    // HANDLES ********************************************************
    handleVariablesBySelectedModel: function( result )
    {
        this.jsonVariables = jQuery.parseJSON( result ).variables;
        this.displayVariables();
    },

    handlePlateformsByVariable: function( result )
    {
        this.jsonPlateforms = jQuery.parseJSON( result ).plateforms;
        this.displayPlateforms();
    },

    // DISPLAYS ********************************************************
    displayModels: function()
    {
        var models = new ListModels( this.jsonSimulations, new Object() );

        models.addOpenListener( jQuery.proxy( this.onSelectModel, this ) );
        this.loadingModels.hide();
        models.display( this.containerModels, "NoModel", interfaceTexts["simulation.noModel"] );

        // Select the first model by default
        if( models && models.array && models.array[0] )
        {
            var modelToSelect = models.getItemById( models.array[0].jsonElement.id );
            this.onSelectModel( modelToSelect.divItem );
            this.selectedModel = modelToSelect.divItem;
        }
    },

    displayVariables: function()
    {
        var variables = new ListVariables( this.jsonVariables, new Object() );

        variables.addOpenListener( jQuery.proxy( this.onSelectVariable, this ) );
        this.loadingVariables.hide();
        variables.display( this.containerVariables, "NoVariable", interfaceTexts["visualization.noVariable"] );
    },

    displayPlateforms: function()
    {
        this.loadingPlateforms.hide();

        if( 0 < this.jsonPlateforms.length )
        {
            var paramSelect = new Object();
            paramSelect.parent = this.containerPlateforms;
            this.selectPlateforms = new Select( paramSelect );
            jQuery.each( this.jsonPlateforms, jQuery.proxy( function ( i, plateform )
            {
                this.selectPlateforms.add( plateform.id, plateform.name );
            }, this ) );
            this.selectPlateforms.selectFirst( false );
        }
        else
            this.containerPlateforms.html( interfaceTexts["simulation.noRealData"] );
    },

    // EVENTS ********************************************************
    onSelectModel: function( objModel )
    {
        if( this.selectedModel && objModel.getId() == this.selectedModel.getId() )
            return;

        // Unselect old model
        if( this.selectedModel )
            this.selectedModel.unselect();

        this.selectedModel = objModel;
        this.selectedModel.select();

        // Unselect old variable
        if( this.selectedVariable )
            this.selectedVariable = false;

        // Change title
        $( "#textareaTitle" ).html( this.selectedModel.getName() + " _ " + this.selectedModel.jsonElement.version );

        this.createSelectDates();
        this.createSelectLevels();
        this.requestVariablesByModel();
        this.testAllFields();
    },

    onSelectVariable: function( objVariable )
    {
        if( this.selectedVariable && objVariable.getId() == this.selectedVariable.getId() )
            return;

        // Unselect old variable
        if( this.selectedVariable )
            this.selectedVariable.unselect();

        this.selectedVariable = objVariable;
        this.selectedVariable.select();

        // Change title
        $( "#textareaTitle" ).html( this.selectedModel.getName() + " _ " + this.selectedModel.jsonElement.version + " - " + this.selectedVariable.getName() );

        // Init plateforms
        this.containerPlateforms.empty();
        $( "#displayRealData" ).attr( "checked", false );

        this.testAllFields();
    },

    onCheckRealData: function()
    {
        if( $( "#displayRealData" ).attr( 'checked' ) )
            this.requestPlateformsByVariable();
        else
            this.containerPlateforms.empty();
    },

    onClickVisualize: function()
    {
        if( this.selectedModel && this.selectedVariable )
        {
            var parameters = "dateBegin=" + this.beginDate
                    + "&dateEnd=" + this.endDate
                    + "&modelId=" + this.selectedModel.getId()
                    + "&variableId=" + this.selectedVariable.getId()
                    + "&variableName=" + this.selectedVariable.getName()
                    + "&dateNumber=" + this.selectDates.getValue()
                    + "&date=" + this.selectDates.getName()
                    + "&level=" + this.selectLevels.getValue()
                    + "&isRasterContour=" + this.selectTypes.getValue()
                    + "&isRealDataToDisplay=" + $( "#displayRealData" ).attr( 'checked' )
                    + "&title=" + encodeURIComponent( $( "#textareaTitle" ).val() );

            if( this.selectPlateforms && this.selectPlateforms.getValue() )
                parameters += "&plateformId=" + this.selectPlateforms.getValue();

            var url = "simulation/simulationPlotEther?" + parameters;
            var $dialog = $( '<div></div>' )
                    .html( '<img src=' + url + ' />' )
                    .dialog( {
                                 autoOpen: false,
                                 title: interfaceTexts["app.title"] + "-" + interfaceTexts["simulation.quicklook"],
                                 minHeight: <%=EtherPlotServiceImpl.getMaxHeight()%>,
                                 minWidth: <%=EtherPlotServiceImpl.getMaxWidth()%> + 28
                             } );
        }
        else
        {
            var $dialog = $( '<div></div>' )
                    .html( '<BR/><center>' + interfaceTexts["visualization.graph.noPlot"] + '</center>' )
                    .dialog( {
                                 autoOpen: false,
                                 title: interfaceTexts["app.title"] + "-" + interfaceTexts["visualization.quicklook"],
                                 height: 50,
                                 width: 400
                             } );
        }

        $dialog.dialog( 'open' );
    },

    // OTHERS ********************************************************
    testAllFields: function()
    {
        if( !this.selectedModel || !this.selectedVariable )
        {
            this.visualizeButton.disable();
            $( "#displayRealData" ).attr( "checked", false );
            $( "#displayRealData" ).attr( "disabled", true );
            return;
        }

        this.visualizeButton.enable();
        $( "#displayRealData" ).attr( "disabled", false );
    }

} );

</script>